package de.intevation.aft;

import java.util.Map;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeSet;

import java.sql.SQLException;
import java.sql.ResultSet;

import org.w3c.dom.Document;

import de.intevation.db.ConnectedStatements;

import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

import org.apache.log4j.Logger;

public class SyncContext
{
    private static Logger log = Logger.getLogger(SyncContext.class);

    protected ConnectedStatements  aftStatements;
    protected ConnectedStatements  flysStatements;
    protected Document             dips;
    protected Map<Long, DIPSGauge> numberToGauge;
    protected Set<TimeInterval>    flysTimeIntervals;

    public SyncContext() {
    }

    public SyncContext(
        ConnectedStatements aftStatements,
        ConnectedStatements flysStatements,
        Document            dips
    ) {
        this.aftStatements  = aftStatements;
        this.flysStatements = flysStatements;
        this.dips           = dips;
    }

    public void init() throws SQLException {
        numberToGauge       = indexByNumber(dips);
        flysTimeIntervals   = loadTimeIntervals();
    }

    public ConnectedStatements getAftStatements() {
        return aftStatements;
    }

    public void setAftStatements(ConnectedStatements aftStatements) {
        this.aftStatements = aftStatements;
    }

    public ConnectedStatements getFlysStatements() {
        return flysStatements;
    }

    public void setFlysStatements(ConnectedStatements flysStatements) {
        this.flysStatements = flysStatements;
    }

    public Document getDips() {
        return dips;
    }

    public void setDips(Document dips) {
        this.dips = dips;
    }

    void close() {
        aftStatements.close();
        flysStatements.close();
    }

    public static Long numberToLong(String s) {
        try {
            return Long.valueOf(s.trim());
        }
        catch (NumberFormatException nfe) {
        }
        return null;
    }

    public Map<Long, DIPSGauge> getDIPSGauges() {
        return numberToGauge;
    }

    protected static Map<Long, DIPSGauge> indexByNumber(Document document) {
        Map<Long, DIPSGauge> map = new HashMap<Long, DIPSGauge>();
        NodeList nodes = document.getElementsByTagName("PEGELSTATION");
        for (int i = nodes.getLength()-1; i >= 0; --i) {
            Element element = (Element)nodes.item(i);
            String numberString = element.getAttribute("NUMMER");
            Long number = numberToLong(numberString);
            if (number != null) {
                DIPSGauge newG = new DIPSGauge(element);
                DIPSGauge oldG = map.put(number, newG);
                if (oldG != null) {
                    log.warn("DIPS: '" + newG.getName() +
                        "' collides with '" + oldG.getName() + 
                        "' on gauge number " + number + ".");
                }
            }
            else {
                log.warn("DIPS: gauge '" + element.getAttribute("NAME") +
                    "' has invalid gauge number '" + numberString + "'");
            }
        }
        return map;
    }

    protected Set<TimeInterval> loadTimeIntervals() throws SQLException {

        boolean debug = log.isDebugEnabled();

        Set<TimeInterval> intervals = new TreeSet<TimeInterval>();
        ResultSet rs = null;
        
        try {
            rs = flysStatements
                .getStatement("select.timeintervals")
                .executeQuery();

            while (rs.next()) {
                int  id    = rs.getInt("id");
                Date start = rs.getDate("start_time");
                Date stop  = rs.getDate("stop_time");

                if (debug) {
                    log.debug("id:    " + id);
                    log.debug("start: " + start);
                    log.debug("stop:  " + stop);
                }

                TimeInterval ti = new TimeInterval(id, start, stop);
                intervals.add(ti);
            }
        }
        finally {
            if (rs != null) {
                rs.close();
            }
        }

        if (debug) {
            log.debug("loaded time intervals: " + intervals.size());
        }

        return intervals;
    }

}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :

